# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 KTH Royal Institute of Technology
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 CERN.
# Copyright (C) 2023 TU Wien.
# Copyright (C) 2024 KTH Royal Institute of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Member Resource API config."""

import marshmallow as ma
from flask_resources import HTTPJSONException, create_error_handler
from invenio_i18n import lazy_gettext as _
from invenio_records_resources.resources import RecordResourceConfig

from ...errors import CommunityDeletedError
from ..errors import AlreadyMemberError, InvalidMemberError


class MemberResourceConfig(RecordResourceConfig):
    """Member resource configuration."""

    blueprint_name = "community_members"
    url_prefix = ""

    routes = {
        "members": "/communities/<pid_value>/members",
        "publicmembers": "/communities/<pid_value>/members/public",
        "invitations": "/communities/<pid_value>/invitations",
    }
    request_view_args = {
        "pid_value": ma.fields.UUID(),
        "member_id": ma.fields.Str(),
    }

    error_handlers = {
        InvalidMemberError: create_error_handler(
            HTTPJSONException(
                code=400,
                description=_("Invalid member specified."),
            )
        ),
        AlreadyMemberError: create_error_handler(
            HTTPJSONException(
                code=400,
                description=_("A member was already added or invited."),
            )
        ),
        CommunityDeletedError: create_error_handler(
            lambda e: (
                HTTPJSONException(
                    code=410, description=_("The record has been deleted.")
                )
            )
        ),
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2024 KTH Royal Institute of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Registry and definition of community roles."""


from dataclasses import dataclass, field

from invenio_i18n import lazy_gettext as _


@dataclass(frozen=True)
class Role:
    """Role class."""

    name: str = ""
    """Name of the role."""

    title: str = ""
    """Title of the role."""

    description: str = ""
    """Brief description of capabilities of the role."""

    can_manage_roles: list = field(default_factory=list)
    """List of other roles that this role can manage."""

    is_owner: bool = False
    """This role is the owner role (only one can exists)."""

    can_manage: bool = False
    """This role has manage permissions."""

    can_curate: bool = False
    """This role has record manage permissions."""

    can_view: bool = False
    """This role has view restricted record permissions."""

    def can_manage_role(self, role_name):
        """Determine if this role can manage the role name."""
        return role_name in self.can_manage_roles

    def __hash__(self):
        """Compute a hash for use with e.g. sets."""
        return self.name.__hash__()


class RoleRegistry:
    """Registry of community roles."""

    def __init__(self, roles_definitions):
        """Initialize the role registry."""
        self._roles = [Role(**role) for role in roles_definitions]

        self._owner = None

        for r in self._roles:
            if r.is_owner:
                assert self._owner is None, _("Only one role be defined as owner.")
                self._owner = r
        assert self._owner is not None, _("One role must be defined as owner.")

    def __contains__(self, key):
        """Determine if key is a valid role id."""
        for role in self._roles:
            if key == role.name:
                return True
        return False

    def __getitem__(self, key):
        """Get a role for a specific key."""
        for role in self._roles:
            if key == role.name:
                return role
        raise KeyError(key)

    def __iter__(self):
        """Iterate list of roles."""
        return iter(self.roles)

    @property
    def roles(self):
        """Get a list of roles."""
        return self._roles

    @property
    def owner_role(self):
        """Get the owner role."""
        return self._owner

    def can(self, action):
        """Returns the roles that can do the action."""
        for role in self.roles:
            if getattr(role, f"can_{action}"):
                yield role

    def manager_roles(self, role_name):
        """Get all roles that can manage members a given role.

        A manager can manage other managers, curators and readers.
        An owner can manage other owners, managers, curators and readers.

        This is used for instance to ensure that a manager cannot invite an
        owner and thereby escalate their privileges.
        """
        allowed_roles = []

        for role in self.roles:
            if role.can_manage_role(role_name):
                allowed_roles.append(role)
        return allowed_roles



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Featured timestamp dumper.

Dumper used to dump/load the featured times of a record to/from an
search body.
"""

from datetime import datetime

from invenio_records.dumpers import SearchDumperExt

from invenio_communities.communities.records.models import CommunityFeatured


class FeaturedDumperExt(SearchDumperExt):
    """Dumper for the featured field."""

    def __init__(self, key="featured"):
        """Initialize the dumper."""
        self.key = key

    def dump(self, record, data):
        """Dump featured entries."""
        now_ = datetime.utcnow()
        future_entries = (
            CommunityFeatured.query.filter(
                CommunityFeatured.community_id == record.id,
                CommunityFeatured.start_date > now_,
            )
            .order_by(CommunityFeatured.start_date.desc())
            .all()
        )

        past_entries = (
            CommunityFeatured.query.filter(
                CommunityFeatured.community_id == record.id,
                CommunityFeatured.start_date <= now_,
            )
            .order_by(CommunityFeatured.start_date.desc())
            .all()
        )

        dumped_field = {}
        dumped_field["future"] = [e.start_date.isoformat() for e in future_entries]
        dumped_field["past"] = [e.start_date.isoformat() for e in past_entries]

        data[self.key] = dumped_field

    def load(self, data, record_cls):
        """Load (remove) indexed data."""
        data.pop(self.key, None)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio communities tasks."""
from celery import shared_task

from invenio_communities.proxies import current_identities_cache


@shared_task
def clear_cache():
    """Clears the cache.

    This is meant to be used to delete the community caches that contain the identity id of the users.
    """
    current_identities_cache.flush()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Mappings for Elasticsearch 7.x."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
# Copyright (C) 2022 Northwestern University.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community views."""
from .components import DefaultCommunityComponents
from .config import CommunityFileServiceConfig, CommunityServiceConfig, SearchOptions
from .service import CommunityService

__all__ = (
    "CommunityService",
    "CommunityServiceConfig",
    "CommunityFileServiceConfig",
    "SearchOptions",
    "DefaultCommunityComponents",
)



#
# This file is part of Invenio.
# Copyright (C) 2023 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Create indeces for bucket_id."""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "72b37bb4119c"
down_revision = "5c68d45c80f0"
branch_labels = ()
depends_on = None


def upgrade():
    """Upgrade database."""
    op.create_index(
        op.f("ix_communities_metadata_bucket_id"),
        "communities_metadata",
        ["bucket_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_communities_files_object_version_id"),
        "communities_files",
        ["object_version_id"],
        unique=False,
    )


def downgrade():
    """Downgrade database."""
    op.drop_index(
        op.f("ix_communities_metadata_bucket_id"), table_name="communities_metadata"
    )
    op.drop_index(
        op.f("ix_communities_files_object_version_id"), table_name="communities_files"
    )
